from .lib import Contactor


__name__ = 'SEC256k2'
__author__ = 'Mohammadreza (Mmdrza.com)'
__version__ = '1.3.6'
__license__ = 'MIT'
__copyright__ = 'Copyright Mmdrza.com @ 2023'
__credits__ = ['Mmdrza']
__description__ = 'SECP256k2 Library for generated and Converted Fast Cryptographic Algorithms Python'
__all__ = [
    'Contactor',
    '__name__',
    '__author__',
    '__version__',
    '__license__',
    '__copyright__',
    '__credits__',
    '__description__'
]
